#include "EPRMDECS.H"

void getread_variables (void)
/* This function fetches user input for the variables that are relevant
to the read menu item. */
{
	/* Declare temporary variable for use below */
	unsigned long temp;
	/* Program calls the relevant data input routine for each
	variable, first outputing a "help" message to the status window
	as appropriate. User input is first stored in the temporary variable
	until it is determined that the input function did not return because
	the <ESC> key was pressed.  If <ESC> was pressed, this function
	returns immediately (exit_code was set to 1 in the input routine).
	Current value of the variable is left unchanged if <ESC> was pressed. */

	/* First turn cursor on for input routines */
	_setcursortype (_NORMALCURSOR);

	/* Fetch details of output file name */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Exact binary image of PROM will be");
	gotoxy (37, 15);
	cprintf ("written to the file");
	textattr (active_input);
	text_input (out_fl_name, 48, 6);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else;

	/* Fetch start address */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Address in PROM at which to start reading");
	gotoxy (37, 15);
	cprintf ("Enter number in decimal or hexadecimal");
	gotoxy (37, 16);
	cprintf ("Start number with h or H for hexadecimal");
	gotoxy (37, 17);
	cprintf ("values, eg h1a0, Hff");
	textattr (active_input);
	temp = longint_input (sstart_addr, 65, 7);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		start_addr = temp;

	/* Fetch number of bytes to read */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Total number of bytes to read");
	gotoxy (37, 15);
	cprintf ("Enter number in decimal or hexadecimal");
	textattr (active_input);
	temp = longint_input (snum_byte, 65, 8);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		num_byte = temp;
	/* Switch cursor off before exiting */
	_setcursortype (_NOCURSOR);
}


void getcompare_variables (void)
/* This function fetches user input for the variables that are relevant
to the compare menu item.  */
{
	/* Declare temporary variable for use below */
	unsigned long temp;
	/* Program calls the relevant data input routine for each
	variable, outputing a "help" message to the status window
	as appropriate. User input is first stored in the temporary variable
	until it is determined that the input function did not return because
	the <ESC> key was pressed.  If <ESC> was pressed, this function
	returns immediately (exit_code was set to 1 in the input routine). */

	/*First turn cursor on for input routines */
	_setcursortype (_NORMALCURSOR);
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("File containing binary data to write to");
	gotoxy (37, 15);
	cprintf ("or compare with the PROM");
	textattr (active_input);
	text_input (in_fl_name, 13, 5);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else;

	/* Fetch source file start address */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Address in source file at which to start");
	gotoxy (37, 15);
	cprintf ("comparison or writing to PROM");
	gotoxy (37, 16);
	cprintf ("Enter number in decimal or hexadecimal");
	gotoxy (37, 17);
	cprintf ("Start number with h or H for hexadecimal");
	gotoxy (37, 18);
	cprintf ("values, eg h1a0, Hff");
	textattr (active_input);
	temp = longint_input (sstart_byte, 17, 7);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		start_byte = temp;

	/* Fetch PROM start address */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Address in PROM at which to start");
	gotoxy (37, 15);
	cprintf ("comparison or write operation");
	gotoxy (37, 16);
	cprintf ("Enter number in decimal or hexadecimal");
	textattr (active_input);
	temp = longint_input (sstart_addr, 17, 8);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		start_addr = temp;

	/* Fetch number of bytes */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Total number of bytes to write or compare");
	gotoxy (37, 15);
	cprintf ("Enter number in decimal or hexadecimal");
	textattr (active_input);
	temp = longint_input (snum_byte, 17, 9);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		num_byte = temp;

	/* Turn cursor off again before exit */
	_setcursortype (_NOCURSOR);
}


void getprogram_variables (int output_colors)
/* This function fetches user input for the variables that are specific
to the program menu item. Note that input for the compare menu items should
first be sought. The function is passed the current screen foreground and
background color settings so that proper value can be passed to routines
called by this function (notably show_device_selection function). */
{
	/* Variable declarations */
	int typed;
	unsigned long temp1;
	float temp2;

	/* Turn cursor on for input routines */
	_setcursortype (_NORMALCURSOR);
	/* Then, call relevant data input routine for first three variables.
	In all cases, input is stored in temporary variable and exit or
	continuation is done depending on whether <ESC> was pressed
	in the input routine (which results in the exit_code being set to 1).
	Message is first output to the screen to provide help to user. */

	/* Fetch write pulse length */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Enter write pulse time in milliseconds");
	gotoxy (37, 15);
	cprintf ("(Numbers with decimal points can be used)");
	gotoxy (37, 16);
	textattr (active_input);
	temp2 = float_input (spulse_len, 21, 11);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		pulse_len = temp2;

	/* Fetch value for number of pulses */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Maximum number of write pulses per address");
	textattr (active_input);
	temp1 = longint_input (swrpulse_num, 21, 12);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		wrpulse_num = temp1;

	/* Fetch final pulse value */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Enter 0 if not required");
	gotoxy (37, 15);
	cprintf ("Otherwise, enter value in milliseconds.");
	gotoxy (37, 16);
	cprintf ("This value is then multiplied by actual");
	gotoxy (37, 17);
	cprintf ("number of write pulses used to calculate");
	gotoxy (37, 18);
	cprintf ("a value in milliseconds for final pulse");
	textattr (active_input);
	temp2 = float_input (sfinal_pulse, 21, 13);
	clear_window (active_input, 13);
	if (exit_code)
		return;
	else
		final_pulse = temp2;

	/* Last input variable is the device type.  Next block of code moves
	the highlight over the relevant option in response to the user
	pressing the up or down arrow key. The device_option variable keeps
	track of current selection.  Once <return> is pressed, the 'while'
	loop is quit and the user is passed to next section of code.

	First, print instructions in the status window, then turn cursor off
	and highlight appropriate option with in a flashing box. */
	textattr (status_window);
	gotoxy (37, 14);
	cprintf ("Different options use different logic");
	gotoxy (37, 15);
	cprintf ("levels on Chip Enable and Vpp pins of");
	gotoxy (37, 16);
	cprintf ("the Programmer.");
	gotoxy (37, 17);
	cprintf ("Customise option allows the user to");
	gotoxy (37, 18);
	cprintf ("manually set the logic levels");
	textattr (active_input);
	_setcursortype (_NOCURSOR);
	show_device_selection ((BLINK + output_colors), (BLINK + output_colors));

	while (ALWAYS)
	{
	/* Up and down arrow keys are sent from the keyboard as two bytes.
	First byte is zero and second byte defines actual key. Check that
	there is a zero first, then fetch second character and action. */
		typed = getch ();
		while (typed == 0)
		{
		typed = getch ();
			/* If down arrow key pressed, move highlight
			to appropriate device option. */
			if (typed == DOWNARROW)
			{
				if (device_option < 4)
				{
					device_option += 1;
					show_device_selection ((BLINK + output_colors), (BLINK + output_colors));
				}
				else
				{
					device_option = 1;
					show_device_selection ((BLINK + output_colors), (BLINK + output_colors));
				}
			}
			/* Or if up arrow key pressed, move highlight
			to appropriate device option. */
			else if (typed == UPARROW)
			{
				if (device_option > 1)
				{
					device_option -= 1;
					show_device_selection ((BLINK + output_colors), (BLINK + output_colors));
				}
				else
				{
					device_option = 4;
					show_device_selection ((BLINK + output_colors), (BLINK + output_colors));
				}
			}
			/* If neither up or down arrow key pressed, reset value
			of 'typed' to a value that will cause a beep below. */
			else
				typed = 45;
		} /* This brace marks end of inner 'while' loop */

		/* If character was not a zero, check for other possibilities
		(or continue here from inner while loop above. */
		if (typed == RETURN)
		{
			/* If <return> typed, use switch construct to set appropriate
			write and verify logic variables, or take other action for
			device_option == 4 ("customise"), before exiting. */

			switch (device_option)
			{
				case 1:
					write_byte = 0x3;
					verify_byte = 0x5;
					standby_a = 0xB;
					standby_b = 0x9;
					break;
				case 2:
					write_byte = 0x3;
					verify_byte = 0x7;
					standby_a = 0xF;
					standby_b = 0xF;
					break;
				case 3:
					write_byte = 0x1;
					verify_byte = 0x5;
					break;
			/* If device_option is "Customise" (option 4), fetch custom write
			logic and return. Cursor is turned on again before user input is
			sought in "custom_bytes ()" so that user can see where input is
			required. Cursor is then turned off again before exit.  The
			"custom_bytes ()" function sets the write and verify logic
			variables. */
				case 4:
					show_device_selection (output_colors, (BLINK + output_colors));
					_setcursortype (_NORMALCURSOR);
					custom_bytes (output_colors);
					_setcursortype (_NOCURSOR);
			}
			/* Set device selection display to non-flasing mode, clear status
			window and return to caller once switch construct is exited. */
			show_device_selection (output_colors, output_colors);
			clear_window (active_input, 13);
			return;
		}
		/* If <escape> pressed, set exit_code flag and return to caller */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return;
		}
		/* Don't beep if you got here by fall through from code above
		after pressing up or down arrow keys */
		else if ((typed != UPARROW) && (typed != DOWNARROW))
		{
			sound (600);
			delay (70);
			nosound ();
		}
		else;
	} /* Go back to start of always repeated 'while' loop */
}


void text_input (char input_string[], int start_column, int line_number)
/* Routine for fetching and converting user string input.  start_column
and line_number are x and y coordinates at which text should start.
Maximum input string length is 32 characters. */
{
	/* Variable declarations */
	char typed;
	int i;
	/* Next section finds and places cursor at the end of any input
	already on screen. Works by checking for the first illegal character
	in variable string and placing cursor at equivalent screen position. */
	for (i = 0; (is_stringchar (input_string[i])) && (i < 33); i++);
	gotoxy ((start_column + i), line_number);
	/* Next "while" loop condition is always true so it is repeatedly
	 executed	until there is an exit from within the loop */
	while (ALWAYS)
	{
		/* Fetch character from keyboard. */
		typed = getch ();
		/* Check if <backspace> and cursor has not been backspaced
		into the prompt string. If backspace was typed, move cursor
		back and delete character at that location */
		if ((typed == 0x8) && (wherex () > start_column))
		{
			cprintf ("\b \b");
		}
		/* Otherwise, check if a legal input character was typed and maximum
		string length has not been exceeded. If so, echo to the screen */
		else if (is_stringchar (typed) && ((wherex () - start_column) <= 32))
			cprintf ("%c", typed);
		/* Otherwise, check if <return> was typed, if so, retrieve the
		string from video ram and convert into a normal character string.
		Note that every second byte contains the video color byte of the
		character location. First "for" loop sets these to null.
		When	done, return to the calling routine */
		else if (typed == RETURN)
		{
			gettext (start_column, line_number, (start_column + 31),
				line_number, input_string);
			/* This "for" loop fills every cell that does not contain a legal
			character with 0. */
			for (i = 0; i < 66; i++)
			{
				if (is_stringchar (input_string[i]));
				else
					input_string[i] = 0;
			}
			/* This "for" loop shifts characters into consecutive cells
			at the start of the array */
			for (i = 2; i < 66;)
			{
				input_string [(i / 2)] = input_string[i];
				input_string[i] = 0;
				i += 2;
			}
			/* Set flag for successful exit */
			exit_code = 0;
			return;
		}
		/* If escape key pressed, exit and flag escape pressed to caller */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return;
		}
		/* Otherwise, check if a function key or <alt> sequence was typed
		These keys return two bytes to the PC, the first being a null
		character. If typed, fetch the second character, discard and
		beep */
		else if (typed == 0)
		{
			getch ();
			sound (600);
			delay (70);
			nosound ();
		}
		/* For all other keys pressed, beep */
		else
		{
			sound (600);
			delay (70);
			nosound ();
		}
	}
}

unsigned long longint_input (char input_string[], int start_column, int line_number)
/* Routine for fetching and converting user input to a long integer.
Routine can handle hexadecimal number input. All such numbers are
recognised by a leading upper or lower case "H". Maximum input string
length is 10 characters.  start_column and line_number are x and y
coordinates at which text should start. */
{
	/* Variable declarations */
	char typed;
	int i;
	/* Next section finds and places cursor at the end of any input
	already on screen. Works by checking for first illegal character
	in variable string and placing cursor at equivalent screen position. */
	for (i = 0; (is_hexvalue (input_string[i])) && (i < 10); i++);
	gotoxy ((start_column + i), line_number);
	/* "While" loop condition is always true so it is repeatedly executed
	until there is an exit from within the loop */
	while (ALWAYS)
	{
		/* Fetch character from keyboard */
		typed = getch ();
		/* Check if <backspace> and cursor has not been backspaced
		into the prompt string. If backspace was typed, move cursor
		to the left one space, overtype with a space and backspace again. */
		if ((typed == 0x8) && (wherex () > start_column))
			cprintf ("\b \b");
		/* Otherwise, check if a legal input character was typed and
		screen limit has not been exceeded. If so, echo input to the screen */
		else if ((is_hexvalue (typed)) && (wherex () < (start_column + 10)))
			cprintf ("%c", typed);
		/* Otherwise, check if <return> was typed, if so, retrieve the
		string from video ram and convert into an unsigned long int.
		Note that every second byte contains the video setting of the
		character location. First "for" loop sets these to null. When
		done, return to the calling routine */
		else if (typed == RETURN)
		{
			gettext (start_column, line_number, (start_column + 9),
				line_number, input_string);
			/* This "for" loop fills every cell that does not contain a
			legal character with 0. */
			for (i = 0; i < 20; i++)
			{
				if (is_hexvalue (input_string[i]));
				else
					input_string[i] = 0;
			}
			/* This "for" loop shifts characters into consecutive cells
			at the start of the array */
			for (i = 2; i < 20;)
			{
				input_string [(i / 2)] = input_string[i];
				input_string[i] = 0;
				i += 2;
			}
		/* Call hex_convert () conversion routine which detects if input
		had a leading h or H and then reads input as a hexadecimal number.
		Otherwise, number is treated as a normal base 10 number */
		return (hex_convert (input_string));
		}
		/* If escape key pressed, exit and flag escape pressed to caller */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return (0);
		}
		/* Otherwise, check if a function key or <alt> sequence was typed
		These keys return two bytes to the PC, the first being a null
		character. If typed, fetch the second character, discard and
		beep */
		else if (typed == 0)
		{
			getch ();
			sound (600);
			delay (70);
			nosound ();
		}
		/* For all other keys pressed, beep */
		else
		{
			sound (600);
			delay (70);
			nosound ();
		}
	}
}

float float_input (char input_string[], int start_column, int line_number)
/* Routine for fetching and converting user input to a float.   start_column
and line_number are x and y coordinates at which text should start.
Maximum input string is 10 characters. */
{
	/* Variable declarations */
	char typed;
	int i;
	float num_value = 0;
	double temp;
	/* Next section finds and places cursor at the end of any input
	already in this field. Works by checking for first illegal character
	in variable string and placing cursor at equivalent screen position. */
	for (i = 0; (is_digit (input_string[i])) && (i < 10); i++);
	gotoxy ((start_column + i), line_number);
	/* "While" loop condition is always true so it is repeatedly executed
	until there is an exit from within the loop */
	while (ALWAYS)
	{
		/* Fetch character from keyboard */
		typed = getch ();
		/* Check if <backspace> and cursor has not been backspaced
		into the prompt string. If backspace was typed, move cursor
		to the left one space, overtype with space and backspace again. */
		if ((typed == 0x8) && (wherex () > start_column))
			cprintf ("\b \b");
		/* Otherwise, check if a legal input character was typed and
		screen limit has not been exceeded. If so, echo to the screen */
		else if ((is_digit (typed)) && (wherex () < (start_column + 9)))
			cprintf ("%c", typed);
		/* Otherwise, check if <return> was typed, if so, retrieve the
		string from video ram and convert into an unsigned long int.
		Note that every second byte contains the video setting of the
		character location. First "for" loop sets these to null. When
		done, return to the calling routine */
		else if (typed == RETURN)
		{
			gettext (start_column, line_number, (start_column + 9),
				line_number, input_string);
			/* This "for" loop fills every cell that does not contain a legal
			character with 0. */
			for (i = 0; i < 20; i++)
			{
					if (is_digit (input_string[i]));
					else
						input_string[i] = 0;
			}
			/* This "for" loop shifts characters into consecutive cells
			at the start of the array */
			for (i = 2; i < 20;)
			{
				input_string [(i / 2)] = input_string[i];
				input_string[i] = 0;
				i += 2;
			}
			/* Convert string to double */
			temp = atof (input_string);
			/* Convert double to float before returning */
			num_value = temp;
			return (num_value);
		}
		/* If escape key pressed, exit and flag escape pressed to caller */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return (0.0);
		}
		/* Otherwise, check if a function key or <alt> sequence was typed
		These keys return two bytes to the PC, the first being a null
		character. If typed, fetch the second character, discard and
		beep */
		else if (typed == 0)
		{
			getch ();
			sound (600);
			delay (70);
			nosound ();
		}
		/* For all other keys pressed, beep */
		else
		{
			sound (600);
			delay (70);
			nosound ();
		}
	}
}


int input_yn (void)
/* Function fetches the character y (or Y) or n (or N) from the keyboard
and echos it to the screen.  If y/Y was typed, function returns 1;
if n/N was typed, function returns zero.  If any other key was typed,
the routine beeps.  Calling function must place cursor at position
where character is to be echoed. */

{
	/* Variable declarations */
	char typed;
	/* While loop always continues, until exit from within the loop  */
	while (ALWAYS)
	{
		/* Fetch character from keyboard */
		typed = getch ();
		/* Check if y/Y or n/N.
		If so, echo to screen and return 1 or 0 respectively.  Otherwise
		beep. */
		if ((typed == 'n') || (typed == 'N'))
		{
			cprintf ("%c", typed);
			return (0);
		}
		else if ((typed == 'y') || (typed == 'Y'))
		{
			cprintf ("%c", typed);
			return (1);
		}
		/* If escape key pressed, exit and flag escape pressed to caller */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return (0);
		}
		else
		{
			sound (600);
			delay (70);
			nosound ();
		}
	}
}



void custom_bytes (int output_colors)
/* Routine to fetch user input and convert to write operation logic byte
that will be sent to the PROM programmer during operation.  This routine
is used when the "Write Operation Logic: Customise" option is selected
under the program menu. */
{
	/* Variable declarations */
	char typed;
	/* Initialise write_byte and verify_byte to appropriate
	values: bit 2 of write byte must always be 0 as programmer hardware
	is incompatible with a write operation while bit 2 (PGM) is high; for
	same reasons, bit 2 of verify_byte must always be high. Bit 0,
	controlling address counter advance, must be kept high, which means
	setting this bit to 1.  Resulting starting values are... */
	write_byte = 1;
	verify_byte = 5;
	/* Fetch user input for CE and Vpp bits and set values of write_byte
	and verify_byte accordingly. If a value different from '0' or '1' is
	returned by input_0_1 () function, this indicates escape key was pressed
	and appropriate exit from this routine should be executed immediately.
	Note that the function "input_0_1 ()" sets exit_code to one if <ESC>
	was pressed.

	First section places cursor at CE row of write_byte and fetches
	user input value, then does conversion.  But check if <ESC> pressed
	first and exit immediately if this is the case.  If input is not a
	legal value, also return to caller immediately. */
	gotoxy (15, 21);
	typed = input_0_1 (output_colors);
	if (typed == '9')
		return;
	else if (typed == '0')
		write_byte += 0;
	else if (typed == '1')
		write_byte += 8;
	else
		return;
	/* Vpp setting of write_byte is now fetched and final value of
	write_byte is calculated */
	gotoxy (15, 22);
	typed = input_0_1 (output_colors);
	if (typed == 9)
		return;
	else if (typed == '0')
		write_byte += 0;
	else if (typed == '1')
		write_byte += 2;
	else
		return;
	/* Similar process to above is repeated, but this time for verify_byte */
	gotoxy (22, 21);
	typed = input_0_1 (output_colors);
	if (typed == 9)
		return;
	else if (typed == '0')
		verify_byte += 0;
	else if (typed == '1')
		verify_byte += 8;
	else
		return;
	gotoxy (22, 22);
	typed = input_0_1 (output_colors);
	if (typed == 9)
		return;
	else if (typed == '0')
		verify_byte += 0;
	else if (typed == '1')
		verify_byte += 2;
	else
		return;
	/* Before exit, standby_a and standby_b are set to appropriate values */
	standby_a = write_byte;
	standby_b = verify_byte;
}


char input_0_1 (int output_colors)
/* Routine to fetch input character 1 or 0 typed at current position of
prompt, echo it to screen and return it to the calling function.  Pressing
any other character results in the PC beeping. The user input is displayed
on the screen with text attribute "output_colors" */
{
	/* Declarations */
	char typed;
	/* Main loop which is repeatedly executed until a return from within */
	while (ALWAYS)
	{
		typed = getch ();
		if ((typed == '0') || (typed == '1'))
		{
			textattr (output_colors);
			cprintf ("%c", typed);
			return (typed);
		}
		/* If escape key pressed, exit and set exit_code to flag that escape
		pressed. */
		else if (typed == ESCAPE)
		{
			exit_code = 1;
			return (9);
		}
		else
		{
			sound (600);
			delay (70);
			nosound ();
		}
	}
}

