#include "EPRMDECS.H"

void prom_write (void)
/* Function to write bytes from source file to EPROM or EEPROM */
{
	/* Declarations */
	int loop_cycles;
	unsigned char in_byte, incr_counta, incr_countb;
	unsigned long non_ff = 0, bytes_done = 0, bytes_todo, wr_error = 0;
	unsigned long write_loop, final_loop, final_loop_cur, i, j, k;
	FILE *source_file;
	/* Set correct output colors for status window and clear window */
	textattr (status_window);
	clear_window (status_window, 12);
	/* Attempt to open file containing source data. Print message and return
	to caller on failure. */
	if ((source_file = fopen (in_fl_name, "rb")) == 0)
	{
		gotoxy (37, 12);
		cprintf ("Error: Couldn't open source file");
		return;
	}
	else;
	/* Check file contains enough data: if yes, continue. Otherwise, exit. */
	if ((filelength (fileno (source_file))) < (start_byte + num_byte))
	{
		fclose (source_file);
		gotoxy (37, 12);
		cprintf ("Error: Source file too short");
		return;
	}
	else;

	/* Calculate write cycle loop counters to
	give requested timing factors (using "timer" assembly language function).
	Also calculate bytes to advance Programmer during programming
	operation. In the latter case, verify_byte is anded to set bit 0 to 0
	(incr_counta) or 1 (incr_countb). */
	write_loop = pulse_len * j_millisec;
	final_loop = final_pulse * j_millisec;
	incr_counta = 0xE & verify_byte;
	incr_countb = 0xF & verify_byte;
	/* Reset programmer address counter, with brief delay in middle to
	ensure Programmer receives reset instruction. */
	outportb (port_control, 0x8);
	delay (1);
	outportb (port_control, 0x5);
	/* Check that all PROM target address locations are set to FF hex, but
	only do test if device option 3 (EEPROM) is not selected. There is
	no point in checking starting contents of EEPROMs as they are simply
	overwritten during programming - they do not have to be specifically
	erased first.

	First, output info to screen. */
	if (device_option != 3)
	{
		gotoxy (37, 12);
		cprintf ("Checking for target addresses in EPROM");
		gotoxy (37, 13);
		cprintf ("set to FF hex...");
		/* Next move programmer address counter to start byte. "timer" call
		is present to provide a brief delay to ensure outportb instructions are
		received and decoded as the Programmer hardware needs a few
		microsecond delay after receipt of any programmer address advance
		instruction. */
		for (i = 0; i < start_addr; i++)
		{
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
			timer (twaste);
		}
		/* Loop to read bytes and check if set to FF. Byte is read in
		two separate nibbles, each of which must have a value of 63 decimal
		if the nibble is F hex. Number of bytes not set to FF hex is
		tallied by incrementing non_ff. Call to "timer" function
		appears between address counter advance instructions to allow
		the Programmer time to decode the first (0x4) instruction. */
		for (i = 0; i < num_byte; i++)
		{
			outportb (port_out, 0x1);
			timer (twaste);
			in_byte = 0xB8 & (inportb (port_in));
			timer (twaste);
			outportb (port_out, 0);
			timer (twaste);
			in_byte += (0xB8 & (inportb (port_in)));
			timer (twaste);
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
			timer (twaste);
			if (in_byte != 112)
				++non_ff;
			else;
		}
		/* Print results */
		clear_window (status_window, 12);
		gotoxy (37, 12);
		cprintf ("Bytes not set to FF: %lu", non_ff);
	}
	/* Do nothing for EEPROM option */
	else;
	/* Check if user wants to continue and respond accordingly */
	textattr (status_window + BLINK);
	gotoxy (37, 14);
	cprintf ("Start writing (y/n)? ");
	if (!(input_yn ()))
	{
		/* Quit this routine, but first close source file, reset text color,
		clear status window and return */
		fclose (source_file);
		textattr (status_window);
		clear_window (status_window, 14);
		return;
	}
	/* Otherwise print "abort" message and place PROM at start address
	for writing. First, prepare status window. */
	else
	{
		textattr (status_window);
		clear_window (status_window, 12);
		gotoxy (37, 12);
		cprintf ("PRESS ANY KEY TO ABORT");
		outportb (port_control, 0x8);
		delay (1);
		outportb (port_control, 0x5);
		timer (twaste);
		for (i = 0; i < start_addr; i++)
		{
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
			timer (twaste);
		}
	}
	/* This is the section of the routine that does actual writing to PROM.
	Main controlling loop carries out write operation for blocks of 16384
	bytes (or fewer). Main loop is repeated appropriate number of times if
	more than 16384 bytes have to be written. Start by placing input file
	pointer to starting byte. */
	fseek (source_file, start_byte, SEEK_SET);
	/* Calculate number of cycles through main loop */
	loop_cycles = (num_byte / 16384) + 1;
	for (i = 0; i < loop_cycles; i++)
	{
		/* Determine bytes still to be written */
		bytes_todo = num_byte - (16384 *i);
		/* Print message and read source file bytes into bdata array.  Up
		to a maximum of 16384, or bytes still left to be written, will be
		put into the array. */
		gotoxy (37, 14);
		cprintf ("Reading data from source file... ");
		for (j = 0; (j < 16384) && (j < bytes_todo); j++)
			bdata [j] = fgetc (source_file);
		clear_window (status_window, 14);
		/* Main loop to write data. There are two versions: one for EPROMs
		(device_option != 3) and the other for EEPROMs
		(device_option == 3). The difference is that for EEPROMs, the
		verify signal is sent straight after the write signal.  This is
		the recommended procedure for SEEQ EEPROMs. The internal circuit
		in these EEPROMs takes care of timing of the actual write
		operation.  The software still times this in order to wait to
		perform its verify operation. */

		if (device_option != 3)
		/* This is the EPROM version of the main write loop */
		{
			for (j = 0; (j < 16384) && (j < bytes_todo); j++)
			{
			/* Send byte to be written to Programmer then send write
			signals. After this, the "timer" call is made to delay for
			the requested time,	then Programmer is set to read (verify)
			mode.  The standby_a and standby_b signals are designed to improve
			compatibility with devices requiring Chip Enable to be sent low to
			perform the write operation. The first (main) "for" loop controls
			exit from the write operation. Exit will occur if the number
			of write pulses (wrpulse_num) per byte is exceeded, or once
			the byte is successfully written as checked by the
			read_byte () routine. Just before the next cycle of the loop,
			"timer" is called to give a delay of about
			25 microseconds or more,	so that the verify_byte instruction
			has time to be decoded by the Programmer hardware before the
			read is performed. */
				for (k = 0; k < wrpulse_num; k++)
				{
					timer (twaste);
					outportb (port_out, bdata [j]);
					outportb (port_control, standby_a);
					timer (twaste);
					outportb (port_control, write_byte);
					timer (write_loop);
					outportb (port_control, standby_b);
					timer (twaste);
					outportb (port_control, verify_byte);
					timer (twaste);
					if (read_byte () == bdata [j])
						break;
					else;
				}
				/* If a final write pulse is required, ie if final_loop
				is not zero, then send a final write pulse to the
				Programmer of length calculated in final_loop from the
				user input final_pulse value (final_loop is calculated
				early in this function and adjusted here for the number of
				write pulses needed to write to the current PROM address).
				Data byte to be written is	output first. */
				if (final_loop)
				{
					final_loop_cur = final_loop * k;
					outportb (port_out, bdata [j]);
					outportb (port_control, standby_a);
					timer (twaste);
					outportb (port_control, write_byte);
					timer (final_loop_cur);
					outportb (port_control, standby_b);
					timer (twaste);
					outportb (port_control, verify_byte);
				}
				else;
				/* Check if byte was successfully written -- if not,
				increment error counter.  The "timer" call is
				present to create a delay so that the verify_byte can
				be decoded by the Programmer hardware, before the read
				is carried out. In all cases, increment bytes_done */
				timer (twaste);
				if (read_byte () != bdata [j])
					++wr_error;
				else;
				++bytes_done;
				/* Increment address counter. Note that the second of the
				two increment instructions (outportb (port_control, incr_countb))
				appears after the "timer" assembly language function call
				ensuring the Programmer has time to receive and decode
				the first instruction (see earlier comment above) */
				outportb (port_control, incr_counta);
				timer (twaste);
				outportb (port_control, incr_countb);
				/* Print progress message every 10 bytes. Also check if user has
				touched keyboard which indicates a wish to abort the operation.
				If so, exit. */
				if (bytes_done % 10);
				else
				{
					gotoxy (37, 14);
					cprintf ("Bytes written: %lu;  Errors: %lu    ", bytes_done, wr_error);
					if (kbhit ())
					{
						getch ();
						fclose (source_file);
						return;
					}
					else;
				}
			}
		}
		else
		/* This is the EEPROM version of the main write loop */
		{
			for (j = 0; (j < 16384) && (j < bytes_todo); j++)
			{
			/* Send byte to be written to Programmer then send write
			signal. Timing is carried out with a call to the "timer"
			assembly language routine, then Programmer is set
			to read (verify) mode. The first (main) "for" loop controls
			exit from the write operation. Exit will occur if the number
			of write pulses (wrpulse_num) per byte is exceeded, or once
			the byte is successfully written as checked by the
			read_byte () routine */
				for (k = 0; k < wrpulse_num; k++)
				{
					timer (twaste);
					outportb (port_out, bdata [j]);
					outportb (port_control, write_byte);
					timer (twaste);
					outportb (port_control, verify_byte);
					timer (write_loop);
					if (read_byte () == bdata [j])
						break;
					else;
				}
				/* If a final write pulse is required, ie if final_loop
				is not zero, then send a final write pulse to the
				Programmer of length calculated in final_loop from the
				user input final_pulse value (final_loop is calculated
				early in this function and adjusted here for the number of
				write pulses needed to write to the current PROM address).
				Data byte to be written is	output first. */
				if (final_loop)
				{
					final_loop_cur = final_loop * k;
					outportb (port_out, bdata [j]);
					outportb (port_control, write_byte);
					timer (twaste);
					outportb (port_control, verify_byte);
					timer (final_loop_cur);
				}
				else;
				/* Check if byte was successfully written -- if not,
				increment	error counter. Increment bytes_done in all cases */
				if (read_byte () != bdata [j])
					++wr_error;
				else;
				++bytes_done;
				/* Increment address counter. Note that the second of the
				two increment instructions (outportb (port_control, incr_countb))
				appears after the call to the "timer" time wasting function
				ensuring the Programmer receives and decodes the first
				instruction (see earlier comment above) */
				outportb (port_control, incr_counta);
				timer (twaste);
				outportb (port_control, incr_countb);
				/* Print progress message every 10 bytes. Also check if user has
				touched keyboard which indicates a wish to abort the operation.
				If so, exit. */
				if (bytes_done % 10);
				else
				{
					gotoxy (37, 14);
					printf ("Bytes written: %lu;  Errors: %lu    ", bytes_done, wr_error);
					if (kbhit ())
					{
						fclose (source_file);
						getch ();
						return;
					}
					else;
				}
			}
		}
	}
	/* Print final message when done and beep. */
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("FINISHED");
	gotoxy (37, 14);
	cprintf ("Bytes written: %lu;  Errors: %lu", bytes_done, wr_error);
	sound (600);
	delay (100);
	sound (800);
	delay (100);
	sound (1000);
	delay (100);
	nosound ();
	fclose (source_file);
}


unsigned char read_byte (void)
/* Routine to read a byte from the Programmer input lines (parallel port address
port_in).  The byte is read as two nibbles which are converted to their
correct value, then combined into a full byte. Byte is read at the address
at which the PROM is currently set. */
{
	/* Variable declarations */
	unsigned int i, nibble;
	unsigned char nibble_read, correct_nibble, temp, full_byte = 0;
	/* Main read loop */
	for (nibble = 0, i = 1; nibble < 2; nibble++, i--)
	{
		/* Start by sending value of i to Programmer data bus. When D0 line
		of the data bus is high ie i = 1, the Programmer is set to output
		the lower nibble of the PROM data byte. When i = 0, D0 of
		Programmer data bus is 0 and upper nibble of PROM data byte is set
		for output. Time waster (call of "timer" function) is placed after
		outport to allow time for decoding of instruction by Programmer. */
		timer (twaste);
		outportb (port_out, i);
		timer (twaste);
		/* Read nibble, then mask unwanted bits with "&" instruction */
		nibble_read = 0xB8 & (inportb (port_in));
		/* Convert input data to correct value.  This is achieved with
		fast bitwise operators and shift operators. The result of the
		following code, in the order executed below, is:
		1	bit 3 of the input data is saved in its current form and
				position (stored in the variable "correct_nibble")
		2	bit 4 of input data is moved into bit 2 of correct_nibble
		3	bit 5 of input data is moved into bit 1 of correct_nibble
		4	bit 7 of input data is moved into bit 0 of correct_nibble
				then inverted with xor instruction.*/
		correct_nibble = nibble_read & 0x8;
		temp = nibble_read & 0x10;
		temp >>= 2;
		correct_nibble |= temp;
		temp = nibble_read & 0x20;
		temp >>= 4;
		correct_nibble |= temp;
		temp = nibble_read & 0x80;
		temp >>= 7;
		correct_nibble |= temp;
		correct_nibble ^= 1;
		/* Combine nibbles into byte -- if upper nibble is being
		processed in this cycle, it is first left shifted to the correct
		position in the byte. */
		if (nibble)
			correct_nibble <<= 4;
		else;
		full_byte += correct_nibble;
	}
	return (full_byte);
}



void manual_read (void)
/* Routine to set Programmer to a requested PROM address so that byte can
be manually read on Programmer LEDs. The value of the byte is also fetched
from the Programmer and displayed on the screen for comparison */
{
	/* Variable declarations */
	unsigned int u_address, l_address;
	unsigned long prom_address, i;
	char sprom_address [20];
	unsigned char byte_read;

	/* First, set up text colors and status_window) */
	textattr (status_window);
	clear_window (status_window, 12);

	/* Next loop is continually repeated until and exit from within */
	while (ALWAYS)
	{
		prom_address = 0;
		sprom_address [0] = 0;
		/* Fetch address to seek */
		gotoxy (37, 12);
		cprintf ("Byte number: ");
		prom_address = longint_input (sprom_address, 50, 12);
		/* Reset programmer address counter */
		outportb (port_control, 0x8);
		delay (1);
		outportb (port_control, 0x5);
		/* Move programmer address counter to requested address, then
		continue executing into next while loop. */
		for (i = 0; i < prom_address; i++)
		{
			timer (twaste);
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
		}
		/* Next loop is continually repeated until a return is effected
		by answering no to "Next address?"  */
		while (ALWAYS)
		{
			/* Read byte from programmer and display */
			byte_read = read_byte ();
			gotoxy (37, 13);
			cprintf ("Byte returned: %02X", byte_read);
			/* Increment address variable and ask whether to read this
			incremented address location */
			prom_address += 1;
			gotoxy (37, 15);
			cprintf ("Next address (y/n)? ");
			if (input_yn ())
			{
				/* Position programmer at next address and
				display address value */
				outportb (port_control, 0x4);
				timer (twaste);
				outportb (port_control, 0x5);
				u_address = prom_address / 0x10000;
				l_address = prom_address % 0x10000;
				clear_window (status_window, 12);
				gotoxy (37, 12);
				cprintf ("Address %X%04X (hex)", u_address, l_address);
				cprintf (", %lu (decimal) ...", prom_address);
				/* Execution now resumes at start of inner "while" loop,
				reading and displaying the byte, then halting at "Next
				address?" prompt */
			}
			else
			{
				/* If answer to "Next address" question in no, return to
				calling routine. */
				clear_window (status_window, 15);
				return;
			}
		}
	}
}


void read_tofile (void)
/* This function reads a selected range of addresses in the PROM and writes
the contents to a user-defined file.  It is called from menu item "Read". */
{
	/* Variable declarations */
	FILE *target_file;
	int loop_cycles;
	unsigned long bytes_todo, i, j, bytes_done = 0;

	/* Set up correct screen color for status window and clear. */
	textattr (status_window);
	clear_window (status_window, 12);

	/* Check if target file already exists and question if user wishes to
	overwrite if so.  Otherwise, simply open for writing. */
	if ((target_file = fopen (out_fl_name, "rb")) != 0)
	{
		fclose (target_file);
		gotoxy (37, 12);
		cprintf ("File already exists. Overwrite (y/n)? ");
		if (!(input_yn ()))
			return;
		else;
	}
	else;

	/* Check if user is ready to continue and respond accordingly */
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("Make sure PROM is inserted and Programmer");
	gotoxy (37, 13);
	cprintf ("is switched on...");
	textattr (status_window + BLINK);
	gotoxy (37, 15);
	cprintf ("Start reading (y/n)? ");
	if (!(input_yn ()))
	{
		/* Quit this routine, but first reset text color and
		clear status window. */
		textattr (status_window);
		clear_window (status_window, 12);
		return;
	}
	else;

	/* If ready, display message, place PROM at start address for reading
	and open output file. */
	textattr (status_window);
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("Preparing to read PROM... please wait");
	if ((target_file = fopen (out_fl_name, "wb")) == 0)
	{
		clear_window (status_window, 12);
		gotoxy (37, 12);
		cprintf ("Unable to open target file");
		return;
	}
	else;
	/* Place PROM at start address for reading -- first reset Programmer
	address counter to zero */
	outportb (port_control, 0x8);
	delay (1);
	outportb (port_control, 0x5);
	/* Advance to start address */
	for (i = 0; i < start_addr; i++)
	{
		timer (twaste);
		outportb (port_control, 0x4);
		timer (twaste);
		outportb (port_control, 0x5);
	}

	/* Main read loop.
	First determine number of cycles of 16384 bytes to read and set main
	loop accordingly (loop_cycles variable). */
	loop_cycles = (num_byte / 16384) + 1;
	for (i = 0; i < loop_cycles; i++)
	{
		/* Determine bytes still to be read */
		bytes_todo = num_byte - (16384 *i);
		/* Loop to read byte from each PROM address location */
		for (j = 0; (j < 16384) && (j < bytes_todo); j++)
		{
			bdata [j] = read_byte ();
			/* Increment counter of number of bytes/addresses read */
			++bytes_done;
			/* Increment address counter. */
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
		}
		/* Write to file */
		for (j = 0; (j < 16384) && (j < bytes_todo); j++)
			fputc (bdata [j], target_file);
	}
	/*  When done, close target file, print results, and make sound. */
	fclose (target_file);
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("%lu bytes successfully read to file:", bytes_done);
	gotoxy (37, 13);
	cprintf ("%s", out_fl_name);
	sound (600);
	delay (100);
	sound (800);
	delay (100);
	sound (1000);
	delay (100);
	nosound ();
}


void compare (void)
/* This routine compares the requested range of addresses in the PROM with
data in the file described in the string variable "in_fl_name. */
{
	/* Variable declarations */
	int loop_cycles;
	unsigned long bytes_done = 0, bytes_todo, errors = 0;
	unsigned long i, j;
	FILE *source_file;

	/* Attempt to open file containing source data. Print message and return
	to caller on failure. */
	textattr (status_window);
	clear_window (status_window, 12);
	if ((source_file = fopen (in_fl_name, "rb")) == 0)
	{
		gotoxy (37, 12);
		cprintf ("Error: Couldn't open source file");
		return;
	}
	else;
	/* Check file contains enough data: if yes, continue. Otherwise, exit. */
	if ((filelength (fileno (source_file))) < (start_byte + num_byte))
	{
		fclose (source_file);
		gotoxy (37, 12);
		cprintf ("Error: Source file too short");
		return;
	}
	else;

	/* Check if user wants to continue and respond accordingly */
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("Ensure PROM is inserted and Programmer");
	gotoxy (37, 13);
	cprintf ("is switched on...");
	textattr (status_window + BLINK);
	gotoxy (37, 15);
	cprintf ("Start the comparison (y/n)? ");
	if (!(input_yn ()))
	{
		/* Quit this routine, but first close source file,
		clear status window and return */
		clear_window (status_window, 12);
		fclose (source_file);
		return;
	}
	else;

	/*Print status message */
	clear_window (status_window, 12);
	gotoxy (37, 12);
	cprintf ("Comparing now. Please wait...");

	/* Reset programmer address counter, with brief delay in middle to
	ensure Programmer receives reset instruction. */
	outportb (port_control, 0x8);
	delay (1);
	outportb (port_control, 0x5);
	/* Next move programmer address counter to start byte. "timer" call
	is present to provide a brief delay to ensure the 0x4 byte is
	received and decoded as the Programmer hardware needs a few
	microsecond delay after receipt of any programmer address advance
	instruction. */
	for (i = 0; i < start_addr; i++)
	{
		timer (twaste);
		outportb (port_control, 0x4);
		timer (twaste);
		outportb (port_control, 0x5);
	}

	/* This is the section of the routine that does actual comparison.
	Main controlling loop carries out the comparison for blocks of 16384
	bytes (or fewer). Main loop is repeated appropriate number of times if
	more than 16384 bytes have to be checked. Start by placing input file
	pointer to starting byte. */
	fseek (source_file, start_byte, SEEK_SET);
	loop_cycles = (num_byte / 16384) + 1;
	for (i = 0; i < loop_cycles; i++)
	{
		/* Determine bytes still to be checked */
		bytes_todo = num_byte - (16384 *i);
		/* Read source file bytes into bdata array.  Up
		to a maximum of 16384, or bytes still left to be checked, will be
		put into the array. */
		for (j = 0; (j < 16384) && (j < bytes_todo); j++)
			bdata [j] = fgetc (source_file);
		/* Read PROM and do comparison */
		for (j = 0; (j < 16384) && (j < bytes_todo); j++)
		{
			if (read_byte () != bdata [j])
				++errors;
			else;
				++bytes_done;
			/* Increment address counter. Note that the second of the
			two increment instructions appears after the "timer" assembly
			language function call ensuring the Programmer has time to
			receive and decode the first instruction (see earlier comment
			above)*/
			outportb (port_control, 0x4);
			timer (twaste);
			outportb (port_control, 0x5);
			/* Print progress message every 10 bytes. */
			if (bytes_done % 10);
			else
			{
				gotoxy (37, 14);
				cprintf ("Bytes checked: %lu;  Errors: %lu    ", bytes_done, errors);
			}
		}
	}
	gotoxy (37, 12);
	cprintf ("COMPARISON COMPLETED:              ");
	gotoxy (37, 14);
	cprintf ("Bytes checked: %lu;  Errors: %lu    ", bytes_done, errors);
}
