#include "EPRMDECS.H"


void paint_window (void)
/* This function paints the status window on the screen.  Most of the
function is tied up with drawing the borders */
{
	/* Variable declarations */
	int i;
	/* Draw a series of blank lines to paint the window */
	textattr (status_window);
	for (i=12; i<24; i++)
	{
		gotoxy (36,i);
		cprintf ("                                           ");
	}
	/* Print information about the parallel port address currently in use. */
	gotoxy (37,23);
	textattr (status_window_perm);
	cprintf ("Base parallel port address: %x", port_out);

	/* Draw borders */
	gotoxy (36,11);
	textattr (status_window);
	putch (CORNER_TL);
	gotoxy (79,11);
	putch (CORNER_TR);
	gotoxy (36,24);
	putch (CORNER_BL);
	gotoxy (79,24);
	putch (CORNER_BR);
	gotoxy (37,11);
	for (i=1; i<43; i++)
		putch (HORIZ_LINE);
	gotoxy (37,24);
	for (i=1; i<43; i++)
		putch (HORIZ_LINE);
	for (i=12; i<24; i++)
	{
		gotoxy (36,i);
		putch (VERT_LINE);
	}
	for (i=12; i<24; i++)
	{
		gotoxy (79,i);
		putch (VERT_LINE);
	}
}

void clear_window (int output_color, int linenumber)
/* This functions clears all but the last line of the status window. It
is passed the current screen colour that is in use by the calling routine
so that it can reset the text color attribute to correct value prior to
exit.  It is also passed the line number from which to start clearing. */
{
	/* Variable declarations */
	int i;
	/* Draw a series of blank lines to paint the window */
	textattr (status_window);
	for (i=linenumber; i<23; i++)
	{
		gotoxy (37,i);
		cprintf ("                                          ");
	}
	textattr (output_color);
}



void compare_prompts (void)
/* This function writes the compare menu prompts to the screen.  The
calling routine should first set the screen color attribute to the
desired setting. */
{
	gotoxy (1,5);
	cprintf ("Source file");
	gotoxy (1,6);
	cprintf ("Start address...");
	gotoxy (4,7);
	cprintf ("Source file");
	gotoxy (4,8);
	cprintf ("EPROM");
	gotoxy (1,9);
	cprintf ("Number of bytes");
}

void program_prompts (void)
/* This function adds to the compare prompts with specific PROM program
option prompts.  The calling routine should first set the screen color
attribute to the desired setting. */
{
	gotoxy (1,10);
	cprintf ("Write operation timing...");
	gotoxy (4,11);
	cprintf ("Write pulse");
	gotoxy (4,12);
	cprintf ("Pulses per byte");
	gotoxy (4,13);
	cprintf ("Final pulse");
	gotoxy (1,14);
	cprintf ("Write operation logic...");
	gotoxy (4,15);
	cprintf ("Standard EPROM single pulse");
	gotoxy (4,16);
	cprintf ("EPROM multiple pulse");
	gotoxy (4,17);
	cprintf ("SEEQ EEPROM");
	gotoxy (4,18);
	cprintf ("Customise...");
	gotoxy (13,19);
	cprintf ("Write  Verify");
	gotoxy (5,21);
	cprintf ("CE pin    ?      ?");
	gotoxy (5,22);
	cprintf ("Vpp pin   ?      ?");
}

void read_prompts (void)
/* This function outputs the read menu prompts.  The calling routine
should first set the screen color attribute to the desired setting. */
{
	gotoxy (46,5);
	cprintf ("File for storing results");
	gotoxy (46,7);
	cprintf ("Start address");
	gotoxy (46,8);
	cprintf ("Number of bytes");
}

void menu_topics (void)
/* This function outputs the menu title blocks.  The calling routine
should first set the screen color attribute to the desired setting. */
{
	gotoxy (5,3);
	cprintf ("DEVICE PROGRAM (ALT-P)");
	gotoxy (7,4);
	cprintf ("OR COMPARE (ALT-C)");
	gotoxy (50,3);
	cprintf ("DEVICE READ (ALT-R)");
	gotoxy (5,25);
	cprintf ("ADDRESS PEEK (ALT-A)");
	gotoxy (68,25);
	cprintf ("QUIT (ALT-Q)");
}

void compare_infields (void)
/* This routine outputs the input fields for the "compare" menu items
to the screen.  It first prints blank spaces to each input field in
order to write the correct screen background color to the field, then
outputs the actual variable using the same screen color attribute.  The
calling routine should first set the screen color attribute to the
desired setting. */
{
	gotoxy (13,5);
	cprintf ("                                ");
	gotoxy (13,5);
	cprintf ("%s", in_fl_name);
	gotoxy (17,7);
	cprintf ("          ");
	gotoxy (17,7);
	cprintf ("%s", sstart_byte);
	gotoxy (17,8);
	cprintf ("          ");
	gotoxy (17,8);
	cprintf ("%s", sstart_addr);
	gotoxy (17,9);
	cprintf ("          ");
	gotoxy (17,9);
	cprintf ("%s", snum_byte);
}

void program_infields (int output_colors)
/* This routine outputs the input fields for the "program" specific menu items
to the screen.  It first prints blank spaces to each input field in
order to write the correct screen background color to the field, then
outputs the actual variable using the same screen color attribute.  The
integer passed to this function is the required screen color attribute. */
{
	textattr (output_colors);
	gotoxy (21,11);
	cprintf ("          ");
	gotoxy (21,11);
	cprintf ("%s", spulse_len);
	gotoxy (21,12);
	cprintf ("          ");
	gotoxy (21,12);
	cprintf ("%s", swrpulse_num);
	gotoxy (21,13);
	cprintf ("          ");
	gotoxy (21,13);
	cprintf ("%s", sfinal_pulse);
	show_device_selection (output_colors, output_colors);
}

void show_device_selection (int output_colors, int logic_colors)
/* This function highlights the appropriate listing from 'Write Operation
Logic...' device selection list.  The item that is highlighted depends on
the value of the global variable 'device_option'.  Most of the function
consists of a switch statement. Just before the switch statement, all items
in the device selection list are set for non-highlighted display.  The
function receives two integer text color attributes.  The first is for
use on the device type option display, the second is the value for
display of the customise logic levels. */
{
	/* First, output all options in non-highlighted appearance */
	textattr (main_screen);
	gotoxy (4,15);
	cprintf ("Standard EPROM single pulse");
	gotoxy (4,16);
	cprintf ("EPROM multiple pulse");
	gotoxy (4,17);
	cprintf ("SEEQ EEPROM");
	gotoxy (4,18);
	cprintf ("Customise...          ");
	/* Now highlight appropriate option depending on the value of
	device_option. The highlight colors are passed to the function by
	the caller. */
	textattr (output_colors);
	switch (device_option)
	{
		case 1:
		gotoxy (4,15);
		cprintf ("Standard EPROM single pulse");
		break;

		case 2:
		gotoxy (4,16);
		cprintf ("EPROM multiple pulse");
		break;

		case 3:
		gotoxy (4,17);
		cprintf ("SEEQ EEPROM");
		break;

		case 4:
		gotoxy (4,18);
		cprintf ("Customise...          ");
		break;
	}
	/* Set text color and background to desired setting for display of the
	customised logic levels data. Only use 'logic_colors' setting if the
	customise selection has been made; otherwise, use non-highlight. */
		if (device_option == 4)
			textattr (logic_colors);
		else
			textattr (main_screen);

	/* Before finishing, fill in values for the 'customise' write and
	verify logic table at the bottom left of the screen. Note that this
	table is filled in with appropriate values irrespective of whether or
	not the customise option is selected.  If 'customise' is not selected,
	the values in this table reflect the logic levels for the actual
	option that is selected.

	Value for CE pin is printed first, then value for Vpp pin. */
		gotoxy (15,21);
		if (write_byte == 1)
		{
			cprintf ("0");
			gotoxy (15,22);
			cprintf ("0");
		}
		else if (write_byte == 3)
		{
			cprintf ("0");
			gotoxy (15,22);
			cprintf ("1");
		}
		else if (write_byte == 9)
		{
			cprintf ("1");
			gotoxy (15,22);
			cprintf ("0");
		}
		else if (write_byte == 11)
		{
			cprintf ("1");
			gotoxy (15,22);
			cprintf ("1");
		}
		else;
		/* Then print values for verify_byte columns */
		gotoxy (22,21);
		if (verify_byte == 5)
		{
			cprintf ("0");
			gotoxy (22,22);
			cprintf ("0");
		}
		else if (verify_byte == 7)
		{
			cprintf ("0");
			gotoxy (22,22);
			cprintf ("1");
		}
		else if (verify_byte == 13)
		{
			cprintf ("1");
			gotoxy (22,22);
			cprintf ("0");
		}
		else if (verify_byte == 15)
		{
			cprintf ("1");
			gotoxy (22,22);
			cprintf ("1");
		}
		else;
	/* Set screen output for proper behaviour of further routines on exit */
	textattr (output_colors);
}


void read_infields (void)
/* This routine outputs the input fields for the "read" menu items
to the screen.  It first prints blank spaces to each input field in
order to write the correct screen background color to the field, then
outputs the actual variable using the same screen color attribute.  The
calling routine should first set the screen color attribute to the
desired setting. */
{
	gotoxy (48,6);
	cprintf ("                                ");
	gotoxy (48,6);
	cprintf ("%s", out_fl_name);
	gotoxy (65,7);
	cprintf ("          ");
	gotoxy (65,7);
	cprintf ("%s", sstart_addr);
	gotoxy (65,8);
	cprintf ("          ");
	gotoxy (65,8);
	cprintf ("%s", snum_byte);
}

