#include "EPRMDECS.H"


void get_dftfile (void)
/* Function to fetch variable values from EPROM.DFT file.  A search is
carried out in the system directory at the time of execution of
this program or the DOS PATH.  EPROM.DFT must be in the same
directory as the EPROM.EXE file, otherwise it will not be found */
{
	/* Declarations */
	char *defaults;
	FILE *dft_file;
	unsigned int temp1, temp2, temp3, temp4;
	/* Search current directory or DOS PATH for the defaults file and
	fill the character array defaults with the full path, after initialising
	first character in "defaults". */
	defaults[0] = 0;
	defaults = searchpath ("eprom.dft");
	/* Open the file if searchpath () returned a value */
	if (defaults[0])
	{
		/* Open default values file in read mode and fetch values */
		dft_file = fopen (defaults, "rt");
		fscanf (dft_file, "%s %s", &in_fl_name, &out_fl_name);
		fscanf (dft_file, "%s %s", &sstart_byte, &sstart_addr);
		fscanf (dft_file, "%s %s", &snum_byte, &swrpulse_num);
		fscanf (dft_file, "%s %s", &spulse_len, &sfinal_pulse);
		fscanf (dft_file, "%d %d %d", &device_option, &temp1, &temp2);
		fscanf (dft_file, "%d %d", &temp3, &temp4);
		/* Convert fetched values into appropriate variable values */
		start_byte = hex_convert (sstart_byte);
		start_addr = hex_convert (sstart_addr);
		num_byte = hex_convert (snum_byte);
		wrpulse_num = hex_convert (swrpulse_num);
		pulse_len = atof (spulse_len);
		final_pulse = atof (sfinal_pulse);
		/* Convert write logic values from the fetched "int" version
		into "unsigned char" format. Close file before exiting. */
		write_byte = temp1;
		verify_byte = temp2;
		standby_a = temp3;
		standby_b = temp4;
		fclose (dft_file);
	}
	else;
}


void write_dftfile (void)
/* Function to write variable values to EPROM.DFT file.  Latter file is
created each time EPROM.EXE is run and is placed in the same directory
as the EPROM.EXE file. Latter must be either in the current
directory or located in DOS path otherwise no EPROM.DFT file is created. */
{
	/* Declarations */
	char *defaults;
	FILE *dft_file;
	int i;
	/* Search current directory or DOS PATH for the program file and
	fill the character array defaults with the full path, after first
	initialising first character in "defaults" with 0. */
	defaults[0] = 0;
	defaults = searchpath ("eprom.exe");
	/* Open the default file if searchpath () returned a value */
	if (defaults[0])
	{
		/* The variable character array "defaults" currently contains
		the name and full path of the .EXE file.  First convert this to the
		.DFT file */
		for (i = 0; defaults [i] !=  0; ++i);
		defaults [i - 3] = 'd';
		defaults [i - 2] = 'f';
		defaults [i - 1] = 't';
		/* Open default values file and write values: one per line. */
		dft_file = fopen (defaults, "wt");
		fprintf (dft_file, "%s\n%s\n", in_fl_name, out_fl_name);
		fprintf (dft_file, "%s\n%s\n", sstart_byte, sstart_addr);
		fprintf (dft_file, "%s\n%s\n", snum_byte, swrpulse_num);
		fprintf (dft_file, "%s\n%s\n", spulse_len, sfinal_pulse);
		fprintf (dft_file, "%d\n", device_option);
		fprintf (dft_file, "%d\n%d\n", write_byte, verify_byte);
		fprintf (dft_file, "%d\n%d\n", standby_a, standby_b);
		fclose (dft_file);
	}
	/* Otherwise write error message. */
	else
		cprintf ("\r\nCouldn't create EPROM.DFT file for storing this session's variables");
}

void t_constant (void)
/* Routine to calculate variable for 1 msec timing on current system. The
routine uses a time wasting loop in t_loop.asm to do the timing. */
{
	/* Declaration of internal variables -- the structure
	"time" is declared in the Borland C library.  Long type is used for
	all integers to avoid overflow error on the right side of
	some expressions below. */
	struct  time t1, t2;
	unsigned long i, j, t_elap, k = 0;
	/* Print timer and system messages. */
	textattr (menu_title + BLINK);
	cprintf ("Adjusting write pulse timer. Please wait....\r\n");
	textattr (main_screen);
	cprintf ("This software prepared for EPROM programmer project in\r\n");
	cprintf ("Electronics Australia Sept/Oct 1993.\r\n");
	cprintf ("Version 2.01    Author: Glenn Pure...  Comments welcomed!\r\n");
	/* Next section of program first determines the number of cycles of
	a loop that need to be executed to last one or several hundredths of
	a second.*/

	/* First set hundredths of a second components of t1 and t2 to current
	time value. */
	gettime(&t1);
	t2.ti_hund = t1.ti_hund;
	/* Next for loop continues until system clock advances to next value of
	hundredths of a second.  Then t1 is initialised to value of t2. */
	for (; t2.ti_hund == t1.ti_hund; gettime(&t2));
	t1 = t2;
	/* Assembly language function "timer" internal loop is set to a repeat
	of 2000 cycles.  Timer is repeated itself until system clock
	increments to next value of hundreds of a second -- latter detected by
	while loop.  Current time is fetched at end of each while loop cycle
	and k in incremented to keep track of total passes through while loop */
	i = 2000;
	while (t1.ti_hund == t2.ti_hund)
	{
		timer (i);
		gettime(&t2);
		k++;
	}
	/* "If" loop checks if seconds changed (incremented) during the above
	while loop.  If it did, value of t2 hundredths of a second is increased
	100 so elapsed time can be properly calculated in t_elap. */
	if (t1.ti_sec != t2.ti_sec)
		t2.ti_hund += 100;
	/* Elapsed time calculated for above loop. */
	t_elap = (t2.ti_hund - t1.ti_hund);
	/* Using the above information, a new call to the "timer" loop function
	can be made which will last about 5 seconds.  This can be used to
	accurately calculate the number of internal loop repeats within "timer"
	to last 1 millisecond.

	First, number of loop cycles required to give approx 5 sec is
	calculated, then "timer" is called using this value, checking the time
	at the start and finish.  */
	j = ((2000 * k) / t_elap) * 500;
	gettime(&t1);
	timer (j);
	gettime(&t2);
	/* Elapsed time for above loop is calculated, but first correcting
	value of t2 seconds in case minutes were incremented.  Value of
	elapsed time is stored in t_elap as hundredths of a second.  */
	if (t2.ti_sec < t1.ti_sec)
		t2.ti_sec += 60;
	t_elap = ((t2.ti_sec - t1.ti_sec) * 100)
	 + (t2.ti_hund - t1.ti_hund);
	 /* loop cycles for 1 millisecond is calculated. */
	 j_millisec = j/(t_elap * 10);
	 /* Print outcome and return to caller */
	textattr (main_screen);
	clrscr ();
	gotoxy (1, 1);
	cprintf ("Best timing accuracy likely on this system approx %3.3f msec (+/- 5\%)", (50./j_millisec));
	return;
}


int is_stringchar (char typed)
/* Routine determines if character sent to function is a valid alpha-
numeric or string character. Function returns 1 if true or 0 if not. */

{
	/* Variable declarations */
	int true_false;
	/* Any character between (numerical value of) ! symbol and ~ symbol) is
	valid. This range includes all upper case and lower case letters and
	all numbers. */
	if ((typed >= '!') && (typed <= '~'))
		true_false = 1;
	else
		true_false = 0;
	return (true_false);
}



int is_hexvalue (char typed)
/* Routine determines if character sent to function is a valid character
representing a hex number (ie h or H, 0-9, a-f, A-F). Function returns 1
if true or 0 if not. */

{
	/* Variable declarations */
	int true_false;
	/* any character between 0 & 9, a & f or A & F or h or H is
	valid. */
	if ((typed >= '0') && (typed <= '9'))
		true_false = 1;
	else if ((typed >= 'a') && (typed <= 'f'))
		true_false = 1;
	else if ((typed >= 'A') && (typed <= 'F'))
		true_false = 1;
	else if ((typed == 'h') || (typed == 'H'))
		true_false = 1;
	else
		true_false = 0;
	return (true_false);
}


int is_digit (char typed)
/* Routine determines if character sent to function is a digit or a decimal
point (.). Function returns 1 if true or 0 if not. */

{
	/* Variable declarations */
	int true_false;
	/* any character between 0 & 9 or "." is valid. */
	if ((typed >= '0') && (typed <= '9'))
		true_false = 1;
	else if (typed == '.')
		true_false = 1;
	else
		true_false = 0;
	return (true_false);
}


unsigned long hex_convert (char input_string[11])
/* Function converts a string into a long int, detecting and converting
hexadecimal numbers by looking for a leading H or h. */
{
	/* Declarations */
	char current_char;
	int i, length;
	unsigned long num_value = 0, factor, temp;
	/* Check if leading character was "H/h": if so, number is
	a hexadecimal value... do conversion */
	if ((input_string[0] == 'h') || (input_string[0] == 'H'))
	{
		length = strlen (input_string);
		/* This loop starts at the right end and works left */
		for (i = length - 1, factor = 1; i > 0; i--)
		{
			/* Convert character to correct numerical value
			or set num_value to 0 if error. Purpose of the subtractions
			is to convert the ASCII value of the character into the
			actual numerical value. */
			current_char = input_string[i];
			if ((current_char >= '0') && (current_char <= '9'))
				current_char -= 48;
			else if ((current_char >= 'A') && (current_char <= 'F'))
				current_char -= 55;
			else if ((current_char >= 'a') && (current_char <= 'f'))
				current_char -= 87;
			else
			{
				num_value = 0;
				return (0);
			}
			/* Place value of input_string[i] into temporary
			long int variable to force coversion to this type
			then convert by multiplying by appropriate factor of 16 */
				temp = current_char;
				num_value += (temp * factor);
				factor *= 16;
		}
	}
	/* Otherwise, value was a decimal number: convert it to
	an unsigned long int using library function atol and return. */
	else
		num_value = atol (input_string);
	return (num_value);
}

